<?php

function ad_views_tables() {
  return array(
    'ads' => array(
      'name' => 'ads',
      'join' => array(
        'left' => array(
          'table' => 'node',
          'field' => 'nid',
        ),
        'right' => array(
          'field' => 'aid',
        ),
      ),
      'fields' => array(
        'adstatus' => array(
          'name' => 'Ad: Status',
          'sortable' => TRUE,
        ),
        'adtype' => array(
          'name' => 'Ad: Type',
          'sortable' => TRUE,
        ),
        'redirect' => array(
          'name' => 'Ad: Redirect URL',
          'handler' => 'ad_handler_field_url',
          'option' => array(
            '#type' => 'select',
            '#options' => array(
              'link' => t('As link'),
              'nolink' => t('Not As link'),
            ),
          ),
        ),
        'activated' => array(
          'name' => 'Ad: Activated',
          'handler' => views_handler_field_dates(),
          'option' => 'string',
          'sortable' => TRUE,
        ),
        'expired' => array(
          'name' => 'Ad: Expired',
          'handler' => views_handler_field_dates(),
          'option' => 'string',
          'sortable' => TRUE,
        ),
        // This feels like a hack, I bet there's a more graceful way.
        'quantity' => array(
          'name' => 'Ad: Quantity',
          'notafield' => TRUE,
          'sortable' => FALSE,
          'option' => 'string',
          'help' => t('When using the <em>Advertisement View Type</em> use this field to specify how many ads to display at the same time.'),
        ),
      ),
      'filters' => array(
        'adstatus' => array(
          'name' => t('Ad: Status'),
          'operator' => 'views_handler_operator_or',
          'list' => 'ad_handler_filter_adstatus',
          'list-type' => 'list',
          'value-type' => 'array',
          'help' => t('Include or exclude ads with the selected status.'),
        ),
        'adtype' => array(
          'name' => t('Ad: Type'),
          'operator' => 'views_handler_operator_or',
          'list' => 'ad_handler_filter_adtypes',
          'list-type' => 'list',
          'value-type' => 'array',
          'help' => t('Include or exclude ads of the selected types.'),
        ),
      ),
    ),
    'ad_owners' => array(
      'name' => 'ad_owners',
      'join' => array(
        'left' => array(
          'table' => 'ads',
          'field' => 'aid',
        ),
        'right' => array(
          'field' => 'aid',
        ),
      ),
      'fields' => array(
        'uid' => array(
          'name' => 'Ad: Owner',
          'sortable' => TRUE,
        ),
      ),
      'filters' => array(
        'uid' => array(
          'name' => t('Ad: Owned by current user'),
          'operator' => array('=' => t('Equals')),
          'value-type' => 'text',
          'handler' => 'ad_handler_filter_current_user',
          'help' => t('Include or exclude ads from the owners groups.'),
        ),
      ),
    ),
    'ad_clicks' => array(
      'name' => 'ad_clicks',
      'join' => array(
        'left' => array(
          'table' => 'ads',
          'field' => 'aid',
        ),
        'right' => array(
          'field' => 'aid',
        ),
      ),
      'fields' => array(
        'uid' => array(
          'name' => 'Ad: Click User Id',
        ),
        'hostname' => array(
          'name' => 'Ad: Click IP Address',
        ),
        'url' => array(
          'name' => 'Ad: Click URL',
          'handler' => 'ad_handler_field_url',
          'option' => array(
            '#type' => 'select',
            '#options' => array(
              'link' => t('As link'),
              'nolink' => t('Not As link'),
            ),
          ),
        ),
        'timestamp' => array(
          'name' => 'Ad: Click Timestamp',
          'handler' => views_handler_field_dates(),
          'option' => 'string',
        ),
      ),
    ),
  );
}

function ad_views_style_plugins() {
  return array(
    'ad' => array(
      'name' => t('Advertisement'),
      'theme' => 'ad_views_view_ads',
      'weight' => -11,
    ),
  );
}

function ad_views_default_views() {
  // Build blocks to display all ads of each type.
  $adtypes = module_invoke_all('adapi', 'type', array());
  foreach ($adtypes as $type) {
    $view = new stdClass();
    $view->name = t('active_@type_ads', array('@type' => $type));
    $view->description = t('Display all active @type ads in a block.', array('@type' => $type));
    $view->access = array();
    $view->view_args_php = '';
    $view->block = TRUE;
    $view->block_title = t('All active @type ads', array('@type' => $type));
    $view->block_header = '';
    $view->block_header_format = '1';
    $view->block_footer = '';
    $view->block_footer_format = '1';
    $view->block_empty = 'No ads found.';
    $view->block_empty_format = '1';
    $view->block_type = 'ad';
    $view->nodes_per_block = '99';
    $view->block_more = FALSE;
    $view->block_use_page_header = FALSE;
    $view->block_use_page_footer = FALSE;
    $view->block_use_page_empty = FALSE;
    $view->sort = array();
    $view->argument = array();
    $view->field = array (
      array (
        'tablename' => 'ads',
        'field' => 'quantity',
        'label' => '',
        'options' => '1',
      ),
    );
    $view->filter = array (
      array (
        'tablename' => 'ads',
        'field' => 'adstatus',
        'operator' => 'OR',
        'options' => '',
        'value' => array (
          0 => 'active',
        ),
      ),
      array (
        'tablename' => 'ads',
        'field' => 'adtype',
        'operator' => 'OR',
        'options' => '',
        'value' => array (
          0 => "$type",
        ),
      ),
    );
    $view->exposed_filter = array();
    $view->requires = array(ads);
    $views[$view->name] = $view;
  }

  // Build blocks to display all ads of each group.
  $groups = ad_handler_filter_group('');
  foreach ($groups as $group) {
    $view = new stdClass();
    $view->name = t('active_@group_group_ads', array('@group' => $group));
    $view->description = t('Display all active @group ads in a block.', array('@group' => $group));
    $view->access = array();
    $view->view_args_php = '';
    $view->block = TRUE;
    $view->block_title = t('All active @group group ads', array('@group' => $group));
    $view->block_header = '';
    $view->block_header_format = '1';
    $view->block_footer = '';
    $view->block_footer_format = '1';
    $view->block_empty = 'No active ads were found in this group.';
    $view->block_empty_format = '1';
    $view->block_type = 'ad';
    $view->nodes_per_block = '99';
    $view->block_more = FALSE;
    $view->block_use_page_header = FALSE;
    $view->block_use_page_footer = FALSE;
    $view->block_use_page_empty = FALSE;
    $view->sort = array();
    $view->argument = array();
    $view->field = array (
      array (
        'tablename' => 'ads',
        'field' => 'quantity',
        'label' => '',
        'options' => '1',
      ),
    );
    $view->exposed_filter = array();
    $view->requires = array(ads);
    $views[$view->name] = $view;
  }

  // Build block to display all active ads.
  $view = new stdClass();
  $view->name = t('all_active_ads');
  $view->description = t('Display all active ads in a block.');
  $view->access = array();
  $view->view_args_php = '';
  $view->block = TRUE;
  $view->block_title = t('All active ads');
  $view->block_header = '';
  $view->block_header_format = '1';
  $view->block_footer = '';
  $view->block_footer_format = '1';
  $view->block_empty = 'No active ads found.';
  $view->block_empty_format = '1';
  $view->block_type = 'ad';
  $view->nodes_per_block = '99';
  $view->block_more = FALSE;
  $view->block_use_page_header = FALSE;
  $view->block_use_page_footer = FALSE;
  $view->block_use_page_empty = FALSE;
  $view->sort = array();
  $view->argument = array();
  $view->field = array (
    array (
      'tablename' => 'ads',
      'field' => 'quantity',
      'label' => '',
      'options' => '1',
    ),
  );
  $view->filter = array (
    array (
      'tablename' => 'ads',
      'field' => 'adstatus',
      'operator' => 'OR',
      'options' => '',
      'value' => array (
        0 => 'active',
      ),
    ),
  );
  $view->exposed_filter = array();
  $view->requires = array(ads);
  $views[$view->name] = $view;

  // Build page to display owner's ads.
  $view = new stdClass();
  $view->name = 'user_ads';
  $view->description = 'All ads owned by user.';
  $view->access = array();
  $view->view_args_php = '';
  $view->menu = TRUE;
  $view->page = TRUE;
  $view->page_title = 'My ads';
  $view->page_header = '';
  $view->page_header_format = '1';
  $view->page_footer = '';
  $view->page_footer_format = '1';
  $view->page_empty = 'You do not own any ads.';
  $view->page_empty_format = '1';
  $view->page_type = 'teaser';
  $view->url = 'ad';
  $view->use_pager = TRUE;
  $view->nodes_per_page = '10';
  $view->sort = array (
    array (
      'tablename' => 'node',
      'field' => 'changed',
      'sortorder' => 'DESC',
      'options' => 'normal',
    ),
  );
  $view->argument = array();
  $view->field = array();
  $view->filter = array (
    array (
      'tablename' => 'ad_owners',
      'field' => 'uid',
      'operator' => '=',
      'options' => '',
      'value' => '',
    ),
  );
  $view->exposed_filter = array();
  $view->requires = array(node);
  $views[$view->name] = $view;

  $status = ad_status_array();
  foreach ($status as $key => $value) {
    $view = new stdClass();
    $view->name = "user_$key". '_ads';
    $view->description = t('All @status ads owned by user.', array('@status' => $key));
    $view->access = array();
    $view->view_args_php = '';
    $view->menu = TRUE;
    $view->page = TRUE;
    $view->page_title = t('My @status ads', array('@status' => $key));
    $view->page_header = '';
    $view->page_header_format = '1';
    $view->page_footer = '';
    $view->page_footer_format = '1';
    $view->page_empty = t('You have no @status ads.', array('@status' => $key));
    $view->page_empty_format = '1';
    $view->page_type = 'teaser';
    $view->url = "ad/$key";
    $view->use_pager = TRUE;
    $view->nodes_per_page = '10';
    $view->sort = array (
      array (
        'tablename' => 'node',
        'field' => 'changed',
        'sortorder' => 'DESC',
        'options' => 'normal',
      ),
    );
    $view->argument = array();
    $view->field = array();
    $view->filter = array (
      array (
        'tablename' => 'ad_owners',
        'field' => 'uid',
        'operator' => '=',
        'options' => '',
        'value' => '',
      ),
      array (
        'tablename' => 'ads',
        'field' => 'adstatus',
        'operator' => 'OR',
        'options' => '',
        'value' => array (
          0 => "$key",
        ),
      ),
    );
    $view->exposed_filter = array();
    $view->requires = array(node, ads);
    $views[$view->name] = $view;
  }

  $view = new stdClass();
  $view->name = 'ActiveAds';
  $view->description = 'All active ads.';
  $view->access = array();
  $view->view_args_php = '';
  $view->page = TRUE;
  $view->page_title = 'All active ads';
  $view->page_header = '';
  $view->page_header_format = '1';
  $view->page_footer = '';
  $view->page_footer_format = '1';
  $view->page_empty = 'There are no active ads.';
  $view->page_empty_format = '1';
  $view->page_type = 'teaser';
  $view->url = 'ad/active';
  $view->use_pager = TRUE;
  $view->nodes_per_page = '10';
  $view->sort = array (
    array (
      'tablename' => 'node',
      'field' => 'changed',
      'sortorder' => 'DESC',
      'options' => 'normal',
    ),
  );
  $view->argument = array();
  $view->field = array();
  $view->filter = array (
    array (
      'tablename' => 'node',
      'field' => 'type',
      'operator' => 'OR',
      'options' => '',
      'value' => array (
        0 => 'ad',
      ),
    ),
    array (
      'tablename' => 'ads',
      'field' => 'adstatus',
      'operator' => 'OR',
      'options' => '',
      'value' => array (
        0 => 'active',
      ),
    ),
  );
  $view->exposed_filter = array();
  $view->requires = array(node, ads);
  $views[$view->name] = $view;
  
  return $views;
}

function theme_ad_views_view_ads($view, $nodes, $type) {
  $nids = array();
  foreach ($nodes as $node) {
    $output .= "$node->nid,";
    $nids[] = $node->nid;
  }
  $options['nids'] = implode(',', $nids);

  foreach ($view->field as $field) {
    if ($field['fullname'] != 'ads.quantity')
      continue;
    $quantity = (int)$field['options'] ? (int)$field['options'] : 1;
    break;
  }

  return ad(NULL, $quantity, $options);
}

function ad_handler_field_url($fieldinfo, $fielddata, $value, $data) {
  if ($fielddata['options'] == 'link') {
    return l($value, $value);
  }
  else {
    return check_plain($value);
  }
}

function ad_handler_filter_group($op) {
  return ad_groups_list(FALSE);
}

function ad_handler_filter_current_user($op, $filterdata, $filterinfo, &$query) {
  global $user;

  $query->ensure_table('ad_owners');
  $query->add_where("ad_owners.uid = %d", $user->uid);
}

function ad_handler_filter_adstatus($op) {
  $adstatus = array();
  $status = ad_status_array();
  foreach ($status as $key => $value) {
    $adstatus[$key] = $key;
  }
  return $adstatus;
}

function ad_handler_filter_adtypes($op) {
  $adtypes = array();
  return drupal_map_assoc(module_invoke_all('adapi', 'type', array()));
}

?>
