<?php
// $Id: adserve.inc,v 1.1.2.5 2007/07/25 20:30:49 jeremy Exp $

/**
 * The main adserve logic.
 */
function adserve_ad($options = array()) {
  static $displayed_count = 0;

  // If no $options are passed in, assume we're using JavaScript.
  if (empty($options)) {
    $method = 'javascript';
  }
  else {
    $method = $options['method'];
    adserve_variable('variable_load', $options);
  }

  adserve_include_drupal();
  adserve_debug();

  adserve_variable('error', FALSE);
  $output = NULL;
  if (adserve_variable('cache') != 'none') {
    $function = 'ad_cache_'. adserve_variable('cache');
    $output = adserve_invoke_cache($function);

  }

  // If cache fails above (or if no cache is specified) we don't use a cache.
  if ($output == NULL) {
    if (adserve_variable('debug')) {
      echo "No cache enabled.<br />\n";
    }
  
    adserve_bootstrap();
    
    if (adserve_variable('nids')) {
      $id = adserve_variable('nids');
      $nids = explode(',', $id);
      $type = 'nids';
      $ids = adserve_served_ids($type);
  
      // Build query to filter on just selected nids.
      $filter = adserve_build_filter($nids, 'n.nid');
      $filter2 = adserve_build_filter($ids, 'a.aid', 'AND', '!=');

      // Randomly determine which ad to display from the specified nids.
      $sql = "SELECT aid, adtype, redirect, activated, maxviews, maxclicks FROM {ads} a WHERE adstatus = 'active' $filter $filter2 ORDER BY RAND() LIMIT ". adserve_variable('quantity');
      $result = db_query($sql);

      if (adserve_variable('debug')) {
        echo "Searching for ad from nid list: $id.<br />\n";
        echo "Query: \"$sql;\"<br />\n";
      }
    }
    else if (adserve_variable('tids')) {
      $id = adserve_variable('tids');
      $tids = explode(',', $id);
      $type = 'tids';
      $ids = adserve_served_ids($type);

      // Build query to filter on just selected tids.
      $filter = adserve_build_filter($tids, 'n.tid');
      $filter2 = adserve_build_filter($ids, 'a.aid', 'AND', '!=');
  
      // Randomly determine which ad to display from the specified tids.
      $sql = "SELECT a.aid, a.adtype, a.redirect, a.activated, a.maxviews, a.maxclicks FROM {ads} a INNER JOIN {term_node} n ON a.aid = n.nid WHERE a.adstatus = 'active' $filter $filter2 ORDER BY RAND() LIMIT ". adserve_variable('quantity');
      $result = db_query($sql);

      if (adserve_variable('debug')) {
        echo "Searching for ad from tid list: $id.<br />\n";
        echo "Query: \"$sql;\"<br />\n";
      }
    }
    else {
      $id = 0;
      $type = 'default';
      $ids = adserve_served_ids($type);
      $filter = adserve_build_filter($ids, 'a.aid', 'AND', '!=');

      // Randomly determine which ad to display from those that do not have
      // any tid assigned to them.
      $sql = "SELECT a.aid, a.adtype, a.redirect, a.activated, a.maxviews, a.maxclicks FROM {ads} a LEFT JOIN {term_node} n ON a.aid = n.nid WHERE a.adstatus = 'active' AND n.tid IS NULL $filter ORDER BY RAND() LIMIT ". adserve_variable('quantity');
      $result = db_query($sql);

      if (adserve_variable('debug')) {
        echo "Searching for ads with no tids.<br />\n";
        echo "Query: \"$sql;\"<br />\n";
      }
    }
      
    $output = '';
    $ads = 0;
    // Include appropriate module for displaying selected ad.
    while ($ad = db_fetch_object($result)) {
      $ids[$ad->aid] = $ad->aid;
      $ads++;
      $module = db_result(db_query("SELECT filename FROM {system} WHERE name = '%s'", "ad_$ad->adtype"));
      if (adserve_variable('debug')) {
        echo "ad: <pre>";
        print_r($ad);
        echo "</pre>";
        echo "Loading module '$module'.<br />\n";
      }
      include_once "$module";
      
      if ($output) {
        // Add a div between ads that themers can use to arrange ads when 
        // displaying more than one at a time.
        $displayed_count++;
        $output .= "<div class=\"advertisement-space\" id=\"$id-$displayed_count\"></div>";
      }
      $output .= module_invoke("ad_$ad->adtype", 'display_ad', $ad);

      // Update the ad's view counter.
      if ($method == 'raw') {
        $output .= ad_display_image($ad);
      }
      else {
        adserve_increment_view($ad);
      }
    }
    adserve_variable("$type-ids", $ids);
    if (empty($ads)) {
      adserve_variable('error', TRUE);
      $output = 'No active ads were found in the '. (empty($nids) ? 'tids' : 'nids') ." '$id'.";
    }
    if (adserve_variable('debug')) {
      echo "Ads displayed: $ads<br />";
    }
  }

  $output = preg_replace('&/@HOSTID___&', '/'. adserve_variable('hostid'), $output);
  if (adserve_variable('error')) {
    $output = "<!-- $output -->";
  }

  switch ($method) {
    case 'javascript':
    default:
      $output = addslashes($output);
      if (!adserve_variable('debug')) {
        // Tell the web browser not to cache this script so the ad refreshes 
        // each time the page is viewed.
        // Expires in the past:
        header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
        // Last loud:
        header('Last-Modified: '. gmdate('D, d M Y H:i:s') .' GMT');
        // HTTP 1.1:
        header('Cache-Control: no-store, no-cache, must-revalidate');
        header('Cache-Control: post-check=0, pre-check=0', false);
        // HTTP 1.0:
        header('Pragma: no-cache');
        // Output is a JavaScript:
        header('Content-Type: application/x-javascript; charset=utf-8');
      }
      print "document.write('$output');";
      exit(0);
    case 'raw':
      chdir(adserve_variable('ad_dir'));
      return $output;
  }
}

/**
 * Retrieve variables from $_GET array or from passed in $value array.
 */
function adserve_variable($variable, $value = NULL) {
  static $variables = NULL;

  // Update the value, if set.
  if (isset($value)) {
    $variables->$variable = $value;
  }

  if (!isset($variables->loaded) || $variable == 'variable_load') {
    if ($variable == 'variable_load') {
      $values['debug'] = $value['debug'];
      $values['c'] = $value['cache'];
      $values['n'] = $value['nids'];
      $values['t'] = $value['tids'];
      $values['k'] = $value['hostid'];
      $values['q'] = $value['quantity'];
      unset($value);
    }
    else {
      $values = $_GET;
    }

    // 'debug' is an integer.
    $variables->debug = isset($values['debug']) ? (int)$values['debug'] : 0;
    // Cache types are comprised of only letters.
    $variables->cache = isset($values['c']) ? preg_replace('/[^a-zA-Z]/', '', $values['c']) : 'none';
    // Nids is an integer or a ",".
    $variables->nids = isset($values['n']) ? preg_replace('/[^0-9,]/', '', $values['n']) : '';
    // Tids is an integer or a ",".
    $variables->tids = isset($values['t']) ? preg_replace('/[^0-9,]/', '', $values['t']) : '';
    // Hostid is an md5() which is comprised of numbers and letters a-f.
    $variables->hostid = isset($values['k']) ? preg_replace('/[^0-9a-f]/', '', $values['k']) : '';
    // Quantity is an integer.
    $variables->quantity = isset($values['q']) ? (int)$values['q'] : 0;
    // Ad ID is an integer.
    $variables->aid = isset($values['a']) ? (int)$values['a'] : 0;

    // Retrieve variables defined by cache plugin, if enabled.
    if ($variables->cache != 'none') {
      $include = $variables->ad_dir ."/cache/$variables->cache/ad_cache_$variables->cache.inc";
      if (file_exists($include)) {
        if ($variables->debug) {
          echo "Attempting to include cache include file '$include'.<br />\n";
        }
        require_once ($include);
      }
      else if ($variables->debug) {
        echo "Failed to find cache include file '$include'.<br />\n";
      }
      $function = "ad_cache_$variables->cache". '_variables';
      if (function_exists($function)) {
        $external_variables = $function();
        foreach ($external_variables as $key => $val) {
          $variables->$key = $val;
        }
      }
    }

    // Set defaults.
    $variables->quantity = $variables->quantity ? $variables->quantity : 1;

    if ($variables->debug) {
      foreach ($variables as $variable => $val) {
        echo "$variable: '$val'<br />\n";
      }
      if ($variables->debug == 1) exit;
    }
    $variables->loaded = TRUE;

    // Override the value, if set during initialization.
    if (isset($value)) {
      $variables->$variable = $value;
    }
  }

  if (isset($variables->$variable)) {
    return $variables->$variable;
  } 
  else {
    return NULL;
  }
}

/**
 * Invoke a function in the enabled cache.
 */
function adserve_invoke_cache($function) {
  /**
   * Ad caches are defined through external modules.  Ad caches are composed 
   * of a module 'ad_cache_TYPE.module' and an include file 
   * 'ad_cache_TYPE.inc' that live in the 'cache/TYPE' subdirectory where 
   * 'TYPE' is replaced with the type of cache.  For example, the included 
   * file cache lives in 'cache/file'.
   *
   * The ad_cache_TYPE.inc file must have a function named ad_cache_TYPE()
   * which is used to display ads.  It can optionally include a function
   * titled ad_cache_TYPE_variables used to extract any necessary
   * variables from the global $_GET array (this can also be used to override
   * values that would normally be set from $_GET).  Any functions used
   * by this code without bootstrapping Drupal should also be in this file.
   *
   * The ad_cache_TYPE.module file should define the drupal _help() hook
   * so the module can be enabled.  It should also define the _adcacheapi()
   * hook allowing for configuration and processing.  Any functions used by
   * this code after bootstrapping Drupal should also be in this module.
   *
   * Refer to cache/file/* for an implementation example.
   */
  $output = '';
  if (function_exists("$function")) {
    $output = $function();
  }
  else if (adserve_variable('debug')) {
    echo "Function '$function' does not exist.<br />\n";
  }
  return $output;
}

/**
 * Include Drupal's bootstrap.inc.
 */
function adserve_include_drupal() {
  adserve_variable('ad_dir', getcwd());

  // Move process to top level Drupal directory so we can perform a bootstrap.
  for ($i = 1; $i <= 20; $i++) {
    if (file_exists('./includes/bootstrap.inc')) {
      break;
    }
    chdir('..');
  }
  adserve_variable('root_dir', getcwd());
  require_once adserve_variable('root_dir') .'/includes/bootstrap.inc';
}

/**
 * Include the necessary files and call the Drupal bootstrap.
 */
function adserve_bootstrap($bootstrap = DRUPAL_BOOTSTRAP_FULL) {
  static $bootstrapped = FALSE;

  if (!$bootstrapped) {
    if (adserve_variable('debug')) {
      echo "Drupal bootstrap '". $bootstrap ."'.<br />\n";
    }

    drupal_bootstrap($bootstrap);
    $bootstrapped = TRUE;
  }
}

/**
 * Increment an ads view counters directly in the database.
 */
function adserve_increment_view($ad) {
  adserve_bootstrap();
  // Update view statistics.
  db_query("UPDATE {ad_statistics} SET count = count + 1 WHERE aid = %d AND action = 'view' AND date = %d AND hostid = '%s'", $ad->aid, date('YmdH'), adserve_variable('hostid'));
  // If column doesn't already exist, we need to add it.
  if (!db_affected_rows()) {
    db_query("INSERT INTO {ad_statistics} (aid, date, action, hostid, count) VALUES(%d, %d, 'view', '%s', 1)", $ad->aid, date('YmdH'), adserve_variable('hostid'));
    // If another process already added this row our INSERT will fail, if 
    // so we still need to increment it so we don't loose a view.
    if (!db_affected_rows()) {
      db_query("UPDATE {ad_statistics} SET count = count + 1 WHERE aid = %d AND action = 'view' AND date = %d AND hostid = '%s'", $ad->aid, date('YmdH'), adserve_variable('hostid'));
    }
  }
  
  // See if we need to perform additional queries.
  if ($ad->maxviews) {
    $views = (int)db_result(db_query("SELECT SUM(count) FROM {ad_statistics} WHERE aid = %d AND action = 'view' AND date >= %d", $ad->aid, date('YmdH', $ad->activated)));
    if ($views >= $ad->maxviews) {
      db_query("UPDATE {ads} SET adstatus = 'expired', autoexpire = 0, autoexpired = %d, expired = %d WHERE aid = %d", time(), time(), $ad->aid);
      ad_statistics_increment($ad->aid, 'autoexpired');
      ad_statistics_increment($ad->aid, 'expired');
    }
  }
  if ($ad->maxclicks) {
    $clicks = (int)db_result(db_query("SELECT SUM(count) FROM {ad_statistics} WHERE aid = %d AND action = 'click' AND date >= %d", $ad->aid, date('YmdH', $ad->activated)));
    if ($clicks >= $ad->maxclicks) {
      db_query("UPDATE {ads} SET adstatus = 'expired', autoexpire = 0, autoexpired = %d, expired = %d WHERE aid = %d", time(), time(), $ad->aid);
      ad_statistics_increment($ad->aid, 'autoexpired');
      ad_statistics_increment($ad->aid, 'expired');
    }
  }
}

/**
 * Display additional debug information.
 */
function adserve_debug() {
  if (adserve_variable('debug')) {
    echo "Root drupal directory detected as '". adserve_variable('root_dir') ."'.<br />\n<br />\n";

    $ad_dir = adserve_variable('ad_dir');
    $files = array("$ad_dir/adserve.php", "$ad_dir/ad.module");
    if (adserve_variable('debug') > 2) {
      $files = array_merge($files, array("$ad_dir/ad.install", "$ad_dir/ad_views.inc"));
    }
    if (adserve_variable('debug') > 3) {
      $files = array_merge($files, array("$ad_dir/image/ad_image.module", "$ad_dir/image/ad_image.install", "$ad_dir/image/ad_image_views.inc", "$ad_dir/text/ad_text.module", "$ad_dir/text/ad_text.install", "$ad_dir/text/ad_text_views.inc", "$ad_dir/embed/ad_embed.module", "$ad_dir/report/ad_report.module", "$ad_dir/notify/ad_notify.module", "$ad_dir/notify/ad_notify.install"));
    }
    foreach ($files as $file) {
      if (!file_exists($file)) {
        echo "Error: '$file' does not exist!<br />\n";
      }
      else if (!is_readable($file)) {
        echo "Error: '$file' is not readable!<br />\n";
      }
      else {
        $fd = fopen($file, 'r');
        while (!feof($fd)) {
          $line = fgets($fd);
          if (substr($line, 0, 5) == "<?php") {
            continue;
          }
          else {
            echo "$file: $line<br />";
            break;
          }
        }
      }
    }
    echo "<br />\n";
  }
}

/**
 * Get array of all $ids served in current group.
 */
function adserve_served_ids($type = 'default') {
  $ids = adserve_variable("$type-ids");

  if (adserve_variable('debug')) {
    echo 'Ads previously served: '. sizeof($ids) .'<br />';
  }

  if (is_array($ids)) {
    return $ids;
  }
  else {
    return array();
  }
}

/**
 * Build a query filter.
 */
function adserve_build_filter($ids, $column, $andor = 'OR', $comparison = '=') {
  $filter = NULL;

  foreach ($ids as $id) {
    if ($filter) {
      $filter .= " $andor $column $comparison $id";
    }
    else {
      $filter = " AND ($column $comparison $id";
    }
  }
  if ($filter) {
    $filter .= ')';
  }

  return $filter;
}
