<?php
// $Id: ad_cache_file.inc,v 1.1.4.13 2007/07/30 00:19:31 jeremy Exp $

/**
 * @file
 * A plug in for the ad.module, providing a file cache mechanism for improved
 * performance when displaying ads.
 *
 * Copyright (c) 2007.
 *  Jeremy Andrews <jeremy@kerneltrap.org>.  All rights reserved.
 */

/**
 * This is the actual cache function called by adserve.php that displays ads
 * without bootstrapping Drupal.
 */
function ad_cache_file() {
  static $displayed_count = 0;

  if (adserve_variable('debug')) {
    echo "Using file cache.<br />\n";
  }

  $cache_file = ad_cache_file_get_lock();
   
  $output = '';
  if ($cache_file) {
    // Read entire cache file into memory.
    $cache = unserialize(fread(adserve_variable('fd'), filesize($cache_file)));
    /**
     * The cache structure looks like this:
     * $cache['ad'][$aid]['display'] = $ad
     * $cache['hostid'][$hostid] = TRUE
     * $cache['ad'][$aid][$hostid][$action][$timestamp] = $counter
     * $cache['last_sync'] = $timestamp
     */

    $hostid = adserve_variable('hostid') ? adserve_variable('hostid') : 'none';
    if ($hostid != 'none' && !isset($cache['hostid'][$hostid])) {
      if (adserve_variable('debug')) {
        echo "Invalid hostid: $hostid<br />\n";
      }
      $output = 'You do not have permission to display ads.';
    }
    else {
      $quantity = adserve_variable('quantity');
      $last_sync = $cache['last_sync'];
      $lifetime = $cache['lifetime'];
      $time = time();
      $timestamp = date('YmdH');
      if (adserve_variable('nids')) {
        $id = adserve_variable('nids');
        $type = 'nids';
        $nids = explode(',', $id);
        $cache_size = sizeof($nids);
      }
      else if (adserve_variable('tids')) {
        $id = adserve_variable('tids');
        $type = 'tids';
        if (!isset($cache['tids'][$id])) {
          $cache['tids'][$id] = array();
          $tids = explode(',', $id);
          foreach ($tids as $tid) {
            if (is_array($cache['tid'][$tid]['aid'])) {
              $cache['tids'][$id] += $cache['tid'][$tid]['aid'];
            }
          }
          // Rebuild keys from 0, cache it for re-use on next ad display
          $cache['tids'][$id] = array_values($cache['tids'][$id]);
        }
        $cache_size = sizeof($cache['tids'][$id]);
        $tids = $cache['tids'][$id];
      }
      else {
        $id = 0;
        $type = 'default';
        $cache_size = sizeof($cache['tid'][0]['aid']);
      }
  
      if (adserve_variable('debug')) {
        echo "Last sync: $last_sync<br />\n";
        echo "Current time: $time<br />\n";
        if ($time - $lifetime >= $last_sync) {
          echo "Will rebuild cache now.<br />\n";
        }
        else {
          $seconds = $last_sync - $time + $lifetime;
          echo "Will rebuild cache in $seconds seconds.<br/>\n";
        }
        echo "Timestamp: $timestamp<br />\n";
        echo "cache_size($cache_size)<br />\n";
      }

      $ids = adserve_variable("$type-ids");
      if ($ids == NULL) {
        $ids = array();
      }
      if (adserve_variable('debug')) {
        echo 'Size of $ids: '. sizeof($ids) .'<br />';
      }
      for ($i = 0; $i < $quantity; $i++) {
        if (adserve_variable('debug')) {
          echo 'Randomly selecting ad: '. ($i + 1) ." of $quantity<br />";
        }
    
        // We can't display more than $cache_size unique ads.
        if ($i >= $cache_size) 
          break;
        $id = FALSE;
    
        $loop = 0;
        while ($id === FALSE || in_array($id, $ids)) {
          // Randomly select a unique banner to display.  We subtract 1 as 
          // arrays start at 0.
          $id = $cache_size > 1 ? mt_rand(0, $cache_size - 1) : 0;
          if (adserve_variable('debug')) {
            echo "Randomly selected ID: $id<br />";
          }
          if (++$loop >= $cache_size)
            break;
        }
    
        $found = FALSE;
        if (isset($nids) && is_array($nids)) {
          $search = $nids;
        }
        else if (isset($tids) && is_array($tids)) {
          $search = $tids;
        }
        else {
          $search = $cache['tid'][0]['aid'];
        }
        if (is_array($search)) {
          $loop = 0;
          while (!$found) {
            if (adserve_variable('debug')) {
              echo "Validating ID $id<br />";
            }
            if ((!in_array($id, $ids)) && 
                 isset($cache['ad'][$search[$id]])) {
              $aid = $search[$id];
              $found = TRUE;
            }
            else {
              // We were passed an invalid nid, walk through all available to
              // find one that is valid.  The ad that comes after a 
              // nonexistent ad will end up being favored by average, but so 
              // it goes...
              if (++$id >= $cache_size) {
                $id = 0;
              }
              // If all passed in nid were invalid, prevent an infinite loop.
              if (++$loop == $cache_size)
                break;
            }
          }
          $ids[] = $id;
          adserve_variable("$type-ids", $ids);
        }
  
        if (!$found) {
          // No ads are available, break out of this loop.
          if (adserve_variable('debug')) {
            echo "No more valid ads found.<br />\n";
          }
          break;
        }
  
        $ad = $cache['ad'][$aid];
  
        if (!empty($output)) {
          // Add a div between ads that themers can use to arrange ads when 
          // displaying more than one at a time.
          $displayed_count++;
          $output .= "<div class=\"advertisement-space\" id=\"$id-$displayed_count\"></div>";
        }
        $output .= $ad['display'];
       
        if (adserve_variable('debug')) {
          echo "Displaying AID: $aid<br />\n";
        }
  
        // Increment view counter.
        if (isset($cache['ad'][$aid][$hostid]) && 
            isset($cache['ad'][$aid][$hostid]['view']) && 
            isset($cache['ad'][$aid][$hostid]['view'][$timestamp])) {
          $cache['ad'][$aid][$hostid]['view'][$timestamp]++;
        }
        else {
          $cache['ad'][$aid][$hostid]['view'][$timestamp] = 1;
        }
      }
  
      // Write updated cache back to file and release the lock.
      $cache = serialize($cache);
      rewind(adserve_variable('fd'));
      ftruncate(adserve_variable('fd'), 0);
      fwrite(adserve_variable('fd'), $cache, strlen($cache));
      flock(adserve_variable('fd'), LOCK_UN);
      fclose(adserve_variable('fd'));
      adserve_variable('fd', '');
  
      // Every $lifetime seconds we flush the cache files to the database.
      if ($last_sync < time() - $lifetime) {
        ad_cache_file_rebuild();
      }
    }
  }
  else {
    $output = 'Configuration error, failed to lock cache file.';
    if (ad_cache_file_rebuild()) {
      // Required function was missing, the file cache must be disabled, so 
      // return nothing allowing adserve.inc to use the default display method.
      return;
    }
  }
  if (empty($output)) {
    adserve_variable('error', TRUE);
    $output = 'No active ads were found in the '. (empty($nids) ? 'tids' : 'nids') ." '$id'.";
    if (adserve_variable('debug')) {
      echo "$output<br />\n";
    }
  }
  return $output;
}

function ad_cache_file_get_lock() {
  static $lock = FALSE;
  static $cache_file = '';

  if ($lock) {
    return $cache_file;
  }

  // We'll loop through all possible cache files until we obtain an 
  // exclusive lock.
  for ($i = 1; $i <= adserve_variable('files'); $i++) {
    // Prefix the filename with a '.' to hide it on Unix systems.
    $cache_file = adserve_variable('path') .'/.'. $i .'.ad.cache';
  
    if (adserve_variable('debug')) {
      echo "Trying cache_file '$cache_file'.<br />\n";
    }
  
    if (!$fd = @fopen($cache_file, 'r+')) {
      if (adserve_variable('debug')) {
        echo "Failed to open cache_file '$cache_file'.<br />\n";
      }
      // We failed to open the cache file, try the next one.
      continue;
    }
    if ($i < adserve_variable('files')) {
      // This isn't the last available cache file so we'll use a 
      // non-blocking lock for best performance.  If we fail to lock this 
      // cache file, we'll quickly move on to the next until we find an 
      // available one.
      if (!flock($fd, LOCK_EX|LOCK_NB)) {
        if (adserve_variable('debug')) {
          echo "Failed to obtain non-blocking lock.<br />\n";
        }
        // We failed to obtain an exclusive lock, close the file and try the
        // next one.
        @fclose($fd);
        continue;
      }
      if (adserve_variable('debug')) {
        echo "Obtained lock.<br />\n";
        $stat = fstat($fd);
        echo 'File size: '. $stat['size'] .'<br />';
      }
      $lock = TRUE;
      break;
    }
    else {
      // This is the last available cache file, we'll use a blocking lock 
      // as we have to wait until we have exclusive write permissions.
      if (!flock($fd, LOCK_EX)) {
        if (adserve_variable('debug')) {
          echo "Failed to obtain blocking lock.<br />\n";
        }
        // A blocking exclusive lock shouldn't ever fail, so something has
        // gone very wrong.  Perhaps the file was deleted out from under us?
        @fclose($fd);
        continue;
      }
      if (adserve_variable('debug')) {
        echo "Obtained lock on final cache file.<br />\n";
      }
      $lock = TRUE;
      break;
    }
  }
  if ($lock) {
    adserve_variable('fd', $fd);
    return $cache_file;
  }
  else {
    return NULL;
  }
}

function ad_cache_file_increment_view() {
  $cache_file = ad_cache_file_get_lock();
  if ($cache_file) {
    $cache = unserialize(fread(adserve_variable('fd'), filesize($cache_file)));

    $hostid = adserve_variable('hostid') ? adserve_variable('hostid') : 'none';
    $aid = adserve_variable('aid');
    if ($hostid != 'none' && !isset($cache['hostid'][$hostid])) {
      if (adserve_variable('debug')) {
        echo "Invalid hostid: $hostid<br />\n";
      }
      $output = 'You do not have permission to display images.';
      return -1;
    }
    else {
      $last_sync = $cache['last_sync'];
      $lifetime = $cache['lifetime'];
      $time = time();
      $timestamp = date('YmdH');

      // Increment view counter.
      if (isset($cache['ad'][$aid][$hostid]) && 
          isset($cache['ad'][$aid][$hostid]['view']) && 
          isset($cache['ad'][$aid][$hostid]['view'][$timestamp])) {
        $cache['ad'][$aid][$hostid]['view'][$timestamp]++;
      }
      else {
        $cache['ad'][$aid][$hostid]['view'][$timestamp] = 1;
      }
  
      // Write updated cache back to file and release the lock.
      $cache = serialize($cache);
      rewind(adserve_variable('fd'));
      ftruncate(adserve_variable('fd'), 0);
      fwrite(adserve_variable('fd'), $cache, strlen($cache));
      flock(adserve_variable('fd'), LOCK_UN);
      fclose(adserve_variable('fd'));
      adserve_variable('fd', '');

      // Every $lifetime seconds we flush the cache files to the database.
      if ($last_sync < time() - $lifetime) {
        ad_cache_file_rebuild();
      }
      return 1;
    }

  }
  return 0;
}

/**
 *  Additional variables required by the filecache.
 */
function ad_cache_file_variables() {
  // paths are comprised of alphanumerics, underscores, dashes, periods and 
  // slashes.
  $variables = array();
  $variables['path'] = isset($_GET['p']) ? preg_replace('/[^_\-\.\/\\0-9a-zA-Z]/', '', $_GET['p']) : '';
  // files is an integer.
  $variables['files'] = isset($_GET['f']) ? (int)$_GET['f'] : 0;
  if ($variables['files'] > 15) {
      echo "Invalid value 'f=". $variables['files'] ."', exiting.<br />\n";
      exit();
  }
  return $variables;
}

/**
 * Bootstrap drupal, then run _ad_build_cache_files() from ad.module which will 
 * rebuild all cache files.
 */
function ad_cache_file_rebuild() {
  adserve_bootstrap();
  if (function_exists('ad_cache_file_build')) {
    ad_cache_file_build();
    return 0;
  }
  else {
    return -1;
  }
}

?>
